<!DOCTYPE html>
<html>
    <head>
        <title>Boldport - TheMatrix</title>
        <script src="{{ url_for('static', filename='jquery-3.1.1.min.js') }}"></script>
    </head>
    <body>
        <div id="header">
            <h1>Boldport - TheMatrix</h1>
        </div>
        {% block content %}{% endblock %}
    </body>
</html>
